import { inspect } from 'node:util';

var isSourcePackage = function (node) {
    var source = node.source.raw;
    if (!source)
        return false;
    return (source.startsWith("'@guardian/src-") ||
        source.startsWith("'@guardian/source-"));
};
var noStarImportsOrExports = {
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow * imports and exports',
            category: 'Possible Problems',
            url: 'https://github.com/guardian/source',
        },
    },
    create: function (context) {
        return {
            ImportDeclaration: function (node) {
                if (!isSourcePackage(node))
                    return;
                var anyNamespaceSpecifiers = !node.specifiers.every(function (s) { return s.type !== 'ImportNamespaceSpecifier'; });
                if (!anyNamespaceSpecifiers)
                    return;
                return context.report({
                    node: node,
                    message: 'Importing * from @guardian/src-* and @guardian/source-* packages is not recommended. Use named imports instead.',
                });
            },
            ExportAllDeclaration: function (node) {
                // e.g. export * from '@guardian/src-foundations'`
                if (!isSourcePackage(node))
                    return;
                return context.report({
                    node: node,
                    message: 'Exporting * from @guardian/src-* and @guardian/source-* packages is not recommended. Use named exports instead.',
                });
            },
        };
    },
};

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var typographyObjChanges = ['body', 'headline', 'textSans', 'titlepiece'];
var removedImports = {
    "'@guardian/source-foundations/utils'": ['InteractionModeEngine'],
    "'@guardian/src-foundations/size/global'": ['remSize', 'remIconSize'],
    "'@guardian/src-foundations/themes'": ['defaultTheme', 'brand', 'brandAlt'],
    "'@guardian/src-helpers'": [
        'storybookBackgrounds',
        'ThemeName',
        'svgBackgroundImage',
    ],
};
var newThemeNames = {
    accordionDefault: 'accordionThemeDefault',
    buttonReaderRevenue: 'buttonThemeReaderRevenue',
    buttonReaderRevenueBrand: 'buttonThemeReaderRevenueBrand',
    buttonReaderRevenueBrandAlt: 'buttonThemeReaderRevenueBrandAlt',
    buttonBrand: 'buttonThemeBrand',
    buttonBrandAlt: 'buttonThemeBrandAlt',
    buttonDefault: 'buttonThemeDefault',
    checkboxBrand: 'checkboxThemeBrand',
    checkboxDefault: 'checkboxThemeDefault',
    choiceCardDefault: 'choiceCardThemeDefault',
    footerBrand: 'footerThemeBrand',
    labelDefault: 'labelThemeDefault',
    labelBrand: 'labelThemeBrand',
    linkBrand: 'linkThemeBrand',
    linkBrandAlt: 'linkThemeBrandAlt',
    linkDefault: 'linkThemeDefault',
    radioBrand: 'radioThemeBrand',
    radioDefault: 'radioThemeDefault',
    selectDefault: 'selectThemeDefault',
    textInputDefault: 'textInputThemeDefault',
    userFeedbackBrand: 'userFeedbackThemeBrand',
    userFeedbackDefault: 'userFeedbackThemeDefault',
};
var capitalise = function (str) {
    return str.charAt(0).toUpperCase() + str.slice(1);
};
var getNewPackage = function (oldPackage) {
    if (oldPackage === "'@guardian/src-foundations/themes'") {
        return "'@guardian/source-react-components'";
    }
    else if (oldPackage.startsWith("'@guardian/src-foundations")) {
        return "'@guardian/source-foundations'";
    }
    else {
        return "'@guardian/source-react-components'";
    }
};
var getRemovedExports = function (node) {
    switch (node.type) {
        case 'ImportDeclaration':
            return node.specifiers.filter(function (i) {
                var source = node.source.raw;
                if (!source || !Object.keys(removedImports).includes(source)) {
                    return false;
                }
                var removedImportsForSource = removedImports[source];
                return (i.type === 'ImportSpecifier' &&
                    removedImportsForSource.includes(i.imported.name));
            });
        case 'ExportNamedDeclaration':
            return node.specifiers.filter(function (i) {
                var _a;
                var source = (_a = node.source) === null || _a === void 0 ? void 0 : _a.raw;
                if (!source || !Object.keys(removedImports).includes(source)) {
                    return false;
                }
                var removedImportsForSource = removedImports[source];
                return removedImportsForSource.includes(i.exported.name);
            });
        case 'ExportAllDeclaration':
            return [];
    }
};
var getImportName = function (i) {
    var imported = i.type === 'ImportSpecifier' ? i.imported.name : i.exported.name;
    var local = i.local.name;
    return imported === local ? imported : "".concat(imported, " as ").concat(local);
};
var getSpecifierName = function (i) {
    return i.type === 'ImportSpecifier' ? i.imported.name : i.exported.name;
};
var getRenameImportFixers = function (node, removedExports, fixer, nodeSource) {
    var _a, _b;
    var fixers = [];
    if (!((_a = node.source) === null || _a === void 0 ? void 0 : _a.raw) || node.type === 'ExportAllDeclaration') {
        return fixers;
    }
    // If anything has been removed then remove it from the fixed import
    // Also add a new line which imports the removed exports from the original source
    if (removedExports.length) {
        for (var _i = 0, removedExports_1 = removedExports; _i < removedExports_1.length; _i++) {
            var i = removedExports_1[_i];
            if (!i.range)
                break;
            // Account for a possible comma after the import (e.g. import {one, two} from 'source')
            var end = i.range[1];
            var comma = nodeSource.slice(end, end + 1);
            fixers.push(fixer.removeRange([i.range[0], comma ? end + 1 : end]));
        }
        var importsArray = [];
        for (var _c = 0, removedExports_2 = removedExports; _c < removedExports_2.length; _c++) {
            var rExport = removedExports_2[_c];
            var newName = getImportName(rExport);
            importsArray.push(newName);
        }
        fixers.push(fixer.insertTextBeforeRange((_b = node.range) !== null && _b !== void 0 ? _b : [0, 0], "import { ".concat(importsArray.join(', '), " } from ").concat(node.source.raw, ";\n")));
    }
    return fixers;
};
var getMessage = function (newPackage, removedExports, node) {
    var _a;
    var importOrExport = node.type.startsWith('Export') ? 'export' : 'import';
    var newPackageMessage = "@guardian/src-* packages are deprecated. ".concat(capitalise(importOrExport), " from ").concat(newPackage, " instead.");
    if (node.type === 'ExportAllDeclaration') {
        return newPackageMessage;
    }
    var renamedExports = [];
    // Some of the typography obj exports have changed name
    if (((_a = node.source) === null || _a === void 0 ? void 0 : _a.raw) === "'@guardian/src-foundations/typography/obj'") {
        for (var _i = 0, _b = node.specifiers; _i < _b.length; _i++) {
            var i = _b[_i];
            if (i.type === 'ImportNamespaceSpecifier' ||
                i.type === 'ImportDefaultSpecifier') {
                continue;
            }
            var name_1 = getSpecifierName(i);
            if (typographyObjChanges.includes(name_1)) {
                renamedExports.push([name_1, "".concat(name_1, "ObjectStyles")]);
            }
        }
    }
    // Some of the theme exports have changed name
    for (var _c = 0, _d = node.specifiers; _c < _d.length; _c++) {
        var i = _d[_c];
        if (i.type === 'ImportNamespaceSpecifier' ||
            i.type === 'ImportDefaultSpecifier') {
            continue;
        }
        var name_2 = getSpecifierName(i);
        if (name_2 in newThemeNames) {
            renamedExports.push([name_2, "".concat(newThemeNames[name_2])]);
        }
    }
    var renamedExportsMessage = '';
    if (renamedExports.length) {
        renamedExportsMessage = "The following export(s) have been renamed [from -> to]: ".concat(renamedExports
            .map(function (_a) {
            var oldName = _a[0], newName = _a[1];
            return "".concat(oldName, " -> ").concat(newName);
        })
            .join(', '));
    }
    if (!removedExports.length) {
        return renamedExportsMessage
            ? "".concat(newPackageMessage, "\n").concat(renamedExportsMessage)
            : newPackageMessage;
    }
    var totalImports = node.specifiers.length;
    var removedExportsArray = [];
    for (var _e = 0, removedExports_3 = removedExports; _e < removedExports_3.length; _e++) {
        var rExport = removedExports_3[_e];
        removedExportsArray.push(rExport.type === 'ImportSpecifier'
            ? rExport.imported.name
            : rExport.exported.name);
    }
    var removedExportsMessage = "The following export(s) have been removed: ".concat(removedExportsArray.join(', '), ".");
    if (totalImports === removedExports.length) {
        return removedExportsMessage;
    }
    else {
        return [newPackageMessage, removedExportsMessage, renamedExportsMessage]
            .filter(function (s) { return !!s; })
            .join('\n');
    }
};
var relevantImportSource = function (importSource, pkg) {
    if (pkg === 'foundations' &&
        importSource.startsWith("'@guardian/src-foundations")) {
        return true;
    }
    if (pkg === 'all' && importSource.startsWith("'@guardian/src-")) {
        return true;
    }
    return false;
};
var createReport = function (context, node, pkg) {
    var _a;
    var importSource = (_a = node.source) === null || _a === void 0 ? void 0 : _a.raw;
    if (!importSource || !relevantImportSource(importSource, pkg))
        return;
    var newPackage = getNewPackage(importSource);
    // Check if the export statement is exporting everything
    // If so, we won't autofix
    if (node.type === 'ExportAllDeclaration') {
        return context.report({
            node: node,
            message: getMessage(newPackage, [], node),
        });
    }
    // Check if the import statement contains any all or default imports
    // If so, we won't autofix
    var nonNamedImports = node.type === 'ImportDeclaration' &&
        !node.specifiers.every(function (s) { return s.type === 'ImportSpecifier'; });
    if (nonNamedImports) {
        return context.report({
            node: node,
            message: getMessage(newPackage, [], node),
        });
    }
    // Some exports are no longer available from the new packages
    // We need to account for these
    var removedExports = getRemovedExports(node);
    var nodeSource = context.getSourceCode().getText(node);
    return context.report({
        node: node,
        message: getMessage(newPackage, removedExports, node),
        fix: function (fixer) {
            var _a, _b;
            // If all of the exports have been removed then don't autofix
            return node.specifiers.length === removedExports.length
                ? null
                : __spreadArray(__spreadArray([], getRenameImportFixers(node, removedExports, fixer, nodeSource), true), [
                    fixer.replaceTextRange((_b = (_a = node.source) === null || _a === void 0 ? void 0 : _a.range) !== null && _b !== void 0 ? _b : [0, 0], newPackage),
                ], false);
        },
    });
};
var validFoundationsImportPath = {
    meta: {
        type: 'problem',
        docs: {
            description: 'Get Source imports from v4 packages',
            category: 'Deprecated',
            url: 'https://github.com/guardian/source',
        },
        fixable: 'code',
        schema: [],
    },
    create: function (context) {
        return {
            ImportDeclaration: function (node) {
                return createReport(context, node, 'foundations');
            },
            ExportNamedDeclaration: function (node) {
                // e.g. export {Props} from '@guardian/src-helpers'
                return createReport(context, node, 'foundations');
            },
            ExportAllDeclaration: function (node) {
                // e.g. export * from '@guardian/src-foundations'`
                return createReport(context, node, 'foundations');
            },
        };
    },
};
console.log(inspect(validFoundationsImportPath, false, null, true /* enable colors */));

var rules = {
    'valid-foundations-import-path': validFoundationsImportPath,
    'no-star-imports-or-exports': noStarImportsOrExports,
};
var configs = {
    recommended: {
        plugins: ['@guardian/source-foundations', 'import'],
        rules: {
            '@guardian/source-foundations/valid-foundations-import-path': 'error',
            '@guardian/source-foundations/no-star-imports-or-exports': 'error',
            'import/no-duplicates': 'error',
        },
    },
};

export { configs, rules };
//# sourceMappingURL=index.js.map
